/* An example of FUNCDEF'ing various Windows OS functions
 * to change the text on the box drawn around various groups.
 *
 * In this example, we'll let REXX Dialog create a simple window,
 * with a PUSH group (containing two buttons) that has a group box
 * around. As each button is pushed, it changes the text in the
 * group box.
 *
 * NOTE: Assumes REXX Dialog is autoloaded.
 */

ADDRESS NULL
NUMERIC DIGITS 10

/* ============== FUNCDEF some needed OS functions ============ */

/* Make things easy for us with WINFUNC option */
OPTIONS "WINFUNC NOSOURCE UNWIND"

/* Trap ERROR, and ask REXX Dialog for ERROR raising */
SIGNAL ON ERROR
SIGNAL ON FAILURE
RXERR('ERROR')

/* Register GetWindowLong(), callable as GetWindowId(), to easily
 * get a window's ID number from its handle
 */
FUNCDEF("GetWindowId", "32u, void, 32", "user32", "GetWindowLong")

/* Register SetDlgItemText() to set the text of a groupbox */
FUNCDEF("SetDlgItemText", "32, void, 32u, str", "user32")





/* ================== Create a REXX Dialog window ===================== */
/* First Group will be some PUSH buttons with a groupbox */
RXTYPE.1 = 'PUSH'
RXFLAGS.1 = 'LABEL|REPORT'
RXLABEL.1 = 'Push 1|Push 2|Groupbox'
RXPOS.1 = '2 10 20'
RXVAL.1 = ''
RX = ''
RXCREATE('RX', 1, 'Main Window', 'MIN|NOCLOSE')







/* ================= Get the handle of the groupbox ================= */
/* Now that the window is open, we can get the ID of the groupbox
 * around the push buttons. To do this, we first need the ID of
 * the main window. The ID of the groupbox will be that main window
 * ID, plus the group number. For example, since the PUSH group
 * is the first group in the window, its group number is 1.
 *
 * To get the main window ID, get the ID of the first control in that
 * group (except for a MENU group) and then subtract the group number
 * multipled by 256.
 */
RXQUERY(, 'HANDLE', 'WindowHandle', 1, 1)
WindowId = GetWindowId(WindowHandle, -12) - (256 * 1)
RXQUERY(, 'HANDLE', 'WindowHandle')







/* ===================== Do user interaction ==================== */
DO FOREVER

   /* NOTE: While we're inside this call to RXMSG(), and some event
    * happens with the above window, then our MyWindowProcess
    * function gets called.
    */
   RXMSG()

   /* Did user click on the CLOSE BOX? */
   IF RXID == '' THEN SIGNAL HALT

   /* He must have clicked on the PUSH button. Change the
    * groupbox text to the same as the button pushed.
    */
   SetDlgItemText(WindowHandle, WindowId + 1 /* PUSH group = #1 */, RXVAL.1)

END













/* ======================= Error Handling ======================= */
SYNTAX:
ERROR:
FAILURE:
    /* NOTE: CONDITION('D') fetches error message. CONDITION('E') fetches the
     * error number. SIGL is the line number where the error occurred.
     */
    CONDITION('M')

HALT:
    RXMSG(,'END')
    RETURN
